//--------------------------------------------------------------------
//Michael Finnegan
//Last update: 29 Feb 2000
//--------------------------------------------------------------------
#include <conio.h>
#include <string.h>

#if !defined  DECOMP_H
#define DECOMP_H
#define MAXSTACK 40
#define MAXSYMB 80
#include <stdio.h>

union ScreenCell
{
	  struct{
											//Each screen cell has these properties
			unsigned colour:    8;     //background and foreground combined
			unsigned decplaces: 4;     //# of decimal places 0 - 9
			unsigned allign:    2;     //0= no alignment, 1=left, 2=center, 3=right
			unsigned commas:    1;     //0= numbers not in commas, 1 = in commas
			unsigned locked:    1;    //if bit 1 is set cell is locked,
			unsigned percent:   1;
			signed  type:       7;   //-1 error, 0 = error, 1 text, 2 number
          }u;
	  char mem[3];

};






struct Token
{
   unsigned opcode: 8;
   char text[19];
};

struct Header           //for reading the headers in Lotus spreadsheet records
{
	unsigned int type;
   unsigned int len;
};

struct Range
{
	int stcol;
	int strow;
	int endcol;
	int endrow;
};

struct Variable
{
   int col;
   int row;
};

struct Format       //The cell's display properties determine
{	                 //what the formula will look like when displayed
	unsigned decimals: 4;   //# of decimal places: bits 0-3
	unsigned fmt_type: 3;   //format type: bits 4-6
	unsigned protect:  1;    //bit 7

};

struct Integer   //for an integer record  7 bytes
{
	Format fmt;
	Variable var;
	int i;

};

struct Double   //for an integer record  13 bytes
{
   Format fmt;
   Variable var;
   double d;

};

struct Label     //5 bytes for label format, col, row
{
   Format fmt;
   Variable var;

};

struct Formula    //for a formula record  15 bytes
{
   Format fmt;     //cell properties
   Variable var;   //the cell name the formula belongs to.
   double d;
   int size;       //the size of the formula

};


struct Symb
{
	unsigned code: 8;  //see formula compiler opcode tables 1-a to 1-d
							 //in Lotus's documentation page 9 to 11 of
							 //Wsff2.txt in the http:www//Simtel.Net archive
							 //code refers to Lotu's raw token code

  unsigned type: 8;   //The type of the symbol. There are 6 types
							 //Type will hold a value between 1 and 6
							 //1. functions  (codes 31 to 97)

							 //2. operators  (codes 8 to 23)

							 //3. ignore or empty symbol
							 //    it refers to codes 24 to 30, 50, 64 to 79
							 //    and 98 to 255

							 //4. operand   (codes are 0, 1, 2, 5)

							 //5. Parantheses  (code 4)

							 //6. end of formula (code 3)


  unsigned argc: 8;    //If its a function, # of args in it.

  union
  {
		double dvalue;   //for double constants
		int ivalue;      //for int contstants
      char text[90];   //for labels and text
		Variable variable;
		Range range;
  }u;
  char appearance[90]; //the appearance of the symbol when displayed
								 //this changes as the symbol is merged into
								 //another. For example if the symbol was "3"
								 //and it was then merged into symbols "*" and "2"
								 //the new appearance would be "3*2"
};


class Decomp
{

	public:
		Decomp();        //constructor
      ~Decomp();   //destructor
      int IsStackEmpty();

		Symb *Pop();
      int Push(Symb *symb);
      int GetAppearanceOfOpCode(char *buf, unsigned char opcode);
      int GetAppearanceOfNumber(char *buf, unsigned char opcode,
                                int inum, double dnum);
      int GetAppearanceOfVariable(char *buf, int col, int row, int src_cell=0);

      int GetAppearanceOfRange(char *buf, int stcol, int strow,
                                int endcol,
                                int endrow, int colon=1);

      int LotusToMatrixFile(char *filename, int colon); 

      int ConvertRawToPostfix(char *rawbuf, int colon=1);
      int ConvertPostfixToInfix(char *inbuf, char *outbuf, int colon=1);
      int WriteMatrixPreliminaryInfo(FILE *outfp);
      int WriteMatrixPreliminaryInfo(FILE *outfp, int cell_count, int lastrow);
      int ConvertLotusPropertiesToMatrix(int cx, int cy, Format cf);

      Format cellfmt;  // properties of the cell the formula is in.
      Symb tempsymb;
      int tos;
      int symblen;
      //Symb stack[MAXSTACK];
      //Symb postfix[MAXSYMB];  //holds the formula in postfix symbol format
      Symb *stack;
      Symb *postfix;


      int cell_col;     //the spreadsheet cell on disk to which
      int cell_row;     //the symbols of the formula belong
     int colwidths[26];
     Range globalRange;     //the global range of the worksheet
     int beg[5];           //beginning pos (zero based) of each line
                            //at the start of a Matrix version 2.0 file
     ScreenCell *sheet;

};
//-----------------------------------------------------------------------
#endif
